@@ -1186,22 +1186,6 @@ libxslt_xsltTransformGetMode(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(py_retval);
 }
 
-PyObject *
-libxslt_xsltTransformGetOutputURI(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
-    PyObject *py_retval;
-    const char * c_retval;
-    xsltTransformContextPtr ctxt;
-    PyObject *pyobj_ctxt;
-
-    if (!PyArg_ParseTuple(args, (char *)"O:xsltTransformGetOutputURI", &pyobj_ctxt))
-        return(NULL);
-    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-
-    c_retval = ctxt->outputFile;
-    py_retval = libxml_charPtrConstWrap((const char *) c_retval);
-    return(py_retval);
-}
-
 PyObject *
 libxslt_xsltFreeExts(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltStylesheetPtr style;
@@ -1217,20 +1201,20 @@ libxslt_xsltFreeExts(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
 }
 
 PyObject *
-libxslt_xsltParseGlobalParam(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+libxslt_xsltCheckExtURI(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    PyObject *py_retval;
+    int c_retval;
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
-    xmlNodePtr cur;
-    PyObject *pyobj_cur;
+    xmlChar * URI;
 
-    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseGlobalParam", &pyobj_style, &pyobj_cur))
+    if (!PyArg_ParseTuple(args, (char *)"Oz:xsltCheckExtURI", &pyobj_style, &URI))
         return(NULL);
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
-    cur = (xmlNodePtr) PyxmlNode_Get(pyobj_cur);
 
-    xsltParseGlobalParam(style, cur);
-    Py_INCREF(Py_None);
-    return(Py_None);
+    c_retval = xsltCheckExtURI(style, URI);
+    py_retval = libxml_intWrap((int) c_retval);
+    return(py_retval);
 }
 
 PyObject *
@@ -1369,19 +1353,20 @@ libxslt_xsltTransformGetInsertNode(PyObject *self ATTRIBUTE_UNUSED, PyObject *ar
 }
 
 PyObject *
-libxslt_xsltStylesheetGetNext(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
-    PyObject *py_retval;
-    xsltStylesheetPtr c_retval;
+libxslt_xsltParseGlobalParam(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
+    xmlNodePtr cur;
+    PyObject *pyobj_cur;
 
-    if (!PyArg_ParseTuple(args, (char *)"O:xsltStylesheetGetNext", &pyobj_style))
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseGlobalParam", &pyobj_style, &pyobj_cur))
         return(NULL);
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
+    cur = (xmlNodePtr) PyxmlNode_Get(pyobj_cur);
 
-    c_retval = style->next;
-    py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
-    return(py_retval);
+    xsltParseGlobalParam(style, cur);
+    Py_INCREF(Py_None);
+    return(Py_None);
 }
 
 PyObject *
@@ -1584,6 +1569,22 @@ libxslt_xsltGetNamespace(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltStylesheetGetNext(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    PyObject *py_retval;
+    xsltStylesheetPtr c_retval;
+    xsltStylesheetPtr style;
+    PyObject *pyobj_style;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltStylesheetGetNext", &pyobj_style))
+        return(NULL);
+    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
+
+    c_retval = style->next;
+    py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltEvalAttrValueTemplate(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
@@ -1654,6 +1655,22 @@ libxslt_xsltRegisterTmpRVT(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltInitAllDocKeys(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    PyObject *py_retval;
+    int c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltInitAllDocKeys", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = xsltInitAllDocKeys(ctxt);
+    py_retval = libxml_intWrap((int) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltPrintErrorContext(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltTransformContextPtr ctxt;
@@ -1809,6 +1826,22 @@ libxslt_xsltTransformGetStyle(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltTransformGetOutputURI(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    PyObject *py_retval;
+    const char * c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltTransformGetOutputURI", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = ctxt->outputFile;
+    py_retval = libxml_charPtrConstWrap((const char *) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltCompileAttr(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltStylesheetPtr style;
